@@ -75,6 +75,12 @@
 /** @brief Number of events per location. */
 #define NUMBER_OF_EVENTS 20000
 
+/** @brief Number of samples in the thumbnail. */
+#define NUMBER_OF_THUMBNAIL_SAMPLES 4
+
+/** @brief Baseline for thumbnail samples. */
+#define THUMBNAIL_BASELINE 1000
+
 /** @brief Archive path */
 #define OTF2_ARCHIVE_PATH "OTF2_Integrity_trace"
 
@@ -345,6 +351,29 @@ create_archive( bool use_sion )
     status = OTF2_Archive_CloseEvtFiles( archive );
     check_status( status, "Close event files." );
 
+    /* Create a thumbnail */
+    uint64_t          refs_to_regions[ 2 ] = { 0, 1 };
+    OTF2_ThumbWriter* thumb_writer         = OTF2_Archive_GetThumbWriter(
+        archive,
+        "region activity",
+        "description",
+        OTF2_THUMBNAIL_TYPE_REGION,
+        NUMBER_OF_THUMBNAIL_SAMPLES,
+        2,     /* number of metrics (i.e., regions) */
+        refs_to_regions );
+    check_pointer( thumb_writer, "Get thumbnail writer." );
+
+    for ( uint64_t sample = 0; sample < NUMBER_OF_THUMBNAIL_SAMPLES; sample++ )
+    {
+        /* ensure different length encodings are used */
+        uint64_t values[ 3 ] = { 250, 750, 250 };
+
+        status = OTF2_ThumbWriter_WriteSample( thumb_writer,
+                                               THUMBNAIL_BASELINE,
+                                               2,
+                                               &values[ sample % 2 ] );
+        check_status( status, "Write thumbnail sample." );
+    }
 
     /* Finalize. */
     OTF2_Archive_Close( archive );
@@ -485,6 +514,66 @@ check_archive( const char* trace )
     status = OTF2_Reader_CloseEvtFiles( reader );
     check_status( status, "Close event files for reading." );
 
+    uint32_t number_of_thumbnails;
+    status = OTF2_Reader_GetNumberOfThumbnails( reader, &number_of_thumbnails );
+    check_status( status, "Get number of thumbnails." );
+
+    if ( number_of_thumbnails == 1 )
+    {
+        OTF2_ThumbReader* thumb_reader = OTF2_Reader_GetThumbReader( reader, 0 );
+        check_pointer( thumb_reader, "Opening thumbnail reader." );
+
+        char*              thumbnail_name;
+        char*              thumbnail_description;
+        OTF2_ThumbnailType thumbnail_type;
+        uint32_t           number_of_samples;
+        uint32_t           number_of_metrics;
+        uint64_t*          refs_to_defs;
+        status = OTF2_ThumbReader_GetHeader( thumb_reader,
+                                             &thumbnail_name,
+                                             &thumbnail_description,
+                                             &thumbnail_type,
+                                             &number_of_samples,
+                                             &number_of_metrics,
+                                             &refs_to_defs );
+        check_status( status, "Get thumbnail header." );
+
+        check_condition( 0 == strcmp( thumbnail_name, "region activity" ),
+                         "Check thumbnail name." );
+        check_condition( 0 == strcmp( thumbnail_description, "description" ),
+                         "Check thumbnail description." );
+        check_condition( thumbnail_type == OTF2_THUMBNAIL_TYPE_REGION,
+                         "Check thumbnail type." );
+        check_condition( number_of_samples == NUMBER_OF_THUMBNAIL_SAMPLES,
+                         "Check sample count." );
+        check_condition( number_of_metrics == 2,
+                         "Check region count." );
+        check_condition( refs_to_defs[ 0 ] == 0 && refs_to_defs[ 1 ] == 1,
+                         "Check region references." );
+
+        for ( uint32_t sample = 0; sample < number_of_samples; sample++ )
+        {
+            uint64_t baseline;
+            uint64_t one_sample[ number_of_metrics ];
+
+            status = OTF2_ThumbReader_ReadSample( thumb_reader,
+                                                  &baseline,
+                                                  number_of_metrics,
+                                                  one_sample );
+            check_status( status, "Reading thumbnail sample." );
+
+            check_condition( baseline == THUMBNAIL_BASELINE,
+                             "Check baseline of sample." );
+
+            for ( uint32_t metric = 0; metric < number_of_metrics; metric++ )
+            {
+                baseline -= one_sample[ metric ];
+            }
+            check_condition( baseline == 0,
+                             "Check values against baseline." );
+        }
+    }
+
     /* Close reader */
     status = OTF2_Reader_Close( reader );
     check_status( status, "Close reader." );
